@@ -4,7 +4,7 @@
 /*                                                                         */
 /*    TrueType character mapping table (cmap) support (body).              */
 /*                                                                         */
-/*  Copyright 2002, 2003, 2004, 2005, 2006 by                              */
+/*  Copyright 2002, 2003, 2004, 2005, 2006, 2007 by                        */
 /*  David Turner, Robert Wilhelm, and Werner Lemberg.                      */
 /*                                                                         */
 /*  This file is part of the FreeType project, and may only be used,       */
@@ -155,6 +155,7 @@
     FT_Byte*  p = cmap->data + 4;
 
 
+    cmap_info->format = 0;
     cmap_info->language = (FT_ULong)TT_PEEK_USHORT( p );
 
     return SFNT_Err_Ok;
@@ -527,6 +528,7 @@
     FT_Byte*  p = cmap->data + 4;
 
 
+    cmap_info->format = 2;
     cmap_info->language = (FT_ULong)TT_PEEK_USHORT( p );
 
     return SFNT_Err_Ok;
@@ -708,7 +710,7 @@
   tt_cmap4_next( TT_CMap4  cmap )
   {
     FT_UInt  charcode;
-    
+
 
     if ( cmap->cur_charcode >= 0xFFFFUL )
       goto Fail;
@@ -862,18 +864,19 @@
       FT_UInt  start, end, offset, n;
       FT_UInt  last_start = 0, last_end = 0;
       FT_Int   delta;
+      FT_Byte*  p_start   = starts;
+      FT_Byte*  p_end     = ends;
+      FT_Byte*  p_delta   = deltas;
+      FT_Byte*  p_offset  = offsets;
 
 
       for ( n = 0; n < num_segs; n++ )
       {
-        p = starts + n * 2;
-        start = TT_PEEK_USHORT( p );
-        p = ends + n * 2;
-        end = TT_PEEK_USHORT( p );
-        p = deltas + n * 2;
-        delta = TT_PEEK_SHORT( p );
-        p = offsets + n * 2;
-        offset = TT_PEEK_USHORT( p );
+        p      = p_offset;
+        start  = TT_NEXT_USHORT( p_start );
+        end    = TT_NEXT_USHORT( p_end );
+        delta  = TT_NEXT_SHORT( p_delta );
+        offset = TT_NEXT_USHORT( p_offset );
 
         if ( start > end )
           FT_INVALID_DATA;
@@ -983,7 +986,7 @@
     for ( ; charcode <= 0xFFFFU; charcode++ )
     {
       FT_Byte*  q;
-      
+
 
       p = cmap->data + 14;               /* ends table   */
       q = cmap->data + 16 + num_segs2;   /* starts table */
@@ -1039,7 +1042,7 @@
     FT_UInt   charcode = *pcharcode;
     FT_UInt   gindex   = 0;
     FT_Byte*  p;
-    
+
 
     p = cmap->data + 6;
     num_segs2 = FT_PAD_FLOOR( TT_PEEK_USHORT( p ), 2 );
@@ -1094,14 +1097,19 @@
           /* search in segments before the current segment */
           for ( i = max ; i > 0; i-- )
           {
-            FT_UInt  prev_end;
+            FT_UInt   prev_end;
+            FT_Byte*  old_p;
 
 
-            p = cmap->data + 14 + ( i - 1 ) * 2;
+            old_p    = p;
+            p        = cmap->data + 14 + ( i - 1 ) * 2;
             prev_end = TT_PEEK_USHORT( p );
 
             if ( charcode > prev_end )
+            {
+              p = old_p;
               break;
+            }
 
             end    = prev_end;
             p     += 2 + num_segs2;
@@ -1297,6 +1305,7 @@
     FT_Byte*  p = cmap->data + 4;
 
 
+    cmap_info->format = 4;
     cmap_info->language = (FT_ULong)TT_PEEK_USHORT( p );
 
     return SFNT_Err_Ok;
@@ -1456,6 +1465,7 @@
     FT_Byte*  p = cmap->data + 4;
 
 
+    cmap_info->format = 6;
     cmap_info->language = (FT_ULong)TT_PEEK_USHORT( p );
 
     return SFNT_Err_Ok;
@@ -1522,7 +1532,7 @@
   /*   NAME        OFFSET         TYPE        DESCRIPTION                  */
   /*                                                                       */
   /*   format      0              USHORT      must be 8                    */
-  /*   reseved     2              USHORT      reserved                     */
+  /*   reserved    2              USHORT      reserved                     */
   /*   length      4              ULONG       length in bytes              */
   /*   language    8              ULONG       Mac language code            */
   /*   is32        12             BYTE[8192]  32-bitness bitmap            */
@@ -1709,6 +1719,7 @@
     FT_Byte*  p = cmap->data + 8;
 
 
+    cmap_info->format = 8;
     cmap_info->language = (FT_ULong)TT_PEEK_ULONG( p );
 
     return SFNT_Err_Ok;
@@ -1857,6 +1868,7 @@
     FT_Byte*  p = cmap->data + 8;
 
 
+    cmap_info->format = 10;
     cmap_info->language = (FT_ULong)TT_PEEK_ULONG( p );
 
     return SFNT_Err_Ok;
@@ -2173,6 +2185,7 @@
     FT_Byte*  p = cmap->data + 8;
 
 
+    cmap_info->format = 12;
     cmap_info->language = (FT_ULong)TT_PEEK_ULONG( p );
 
     return SFNT_Err_Ok;
@@ -2273,7 +2286,7 @@
 
       if ( offset && offset <= face->cmap_size - 2 )
       {
-        FT_Byte*                       cmap   = table + offset;
+        FT_Byte* volatile              cmap   = table + offset;
         volatile FT_UInt               format = TT_PEEK_USHORT( cmap );
         const TT_CMap_Class* volatile  pclazz = tt_cmap_classes;
         TT_CMap_Class volatile         clazz;
@@ -2293,7 +2306,8 @@
 
             valid.num_glyphs = (FT_UInt)face->max_profile.numGlyphs;
 
-            if ( ft_setjmp( FT_VALIDATOR( &valid )->jump_buffer ) == 0 )
+            if ( ft_setjmp(
+              *((ft_jmp_buf*)&FT_VALIDATOR( &valid )->jump_buffer) ) == 0 )
             {
               /* validate this cmap sub-table */
               error = clazz->validate( cmap, FT_VALIDATOR( &valid ) );
